/*******************************************************************************
 * University of Illinois/NCSA
 * Open Source License
 *
 * Copyright (c) 2010 , NCSA.  All rights reserved.
 *
 * Developed by:
 * Cyberenvironments and Technologies (CET)
 * http://cet.ncsa.illinois.edu/
 *
 * National Center for Supercomputing Applications (NCSA)
 * http://www.ncsa.illinois.edu/
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimers.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in the
 *   documentation and/or other materials provided with the distribution.
 * - Neither the names of CET, University of Illinois/NCSA, nor the names
 *   of its contributors may be used to endorse or promote products
 *   derived from this Software without specific prior written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *******************************************************************************/
package edu.uiuc.ncsa.cet.bean;

import java.io.Serializable;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 * A bean to associate any kind of blob (artifact in general) with any object
 * (document, person, tag etc). Needed as multiple blobs can be associated with
 * same object. Properties of blob bean helps to distinguish between different
 * blobs associated with same object.
 * 
 * @author kkotwani
 */
public class BlobBean extends CETBean implements Serializable {

	// Blob is attached separately as inputstream to the beansession.

	/**
	 * 
	 */
	private static final long serialVersionUID = 7799088490555096633L;

	/**
	 * usually a file name. Store original name (with extension) even though
	 * user changes title at file bean level. Will help to download file with
	 * correct extension even if user forgets to specify it during upload.
	 */
	private String name = "";

	/**
	 * MIME type helps to distinguish between different format of blobs.
	 */
	private String mimeType = "";

	/**
	 * Types defined based on DCMIType vocabulary
	 */
	private String dcmiType = "";

	/**
	 * size of blob
	 */
	private long size = 0;

	/** Description of the blob */
	private String description = "";

	/** last modified date */
	private Date lastModifiedDate = new Date();

	/** creator of the blob */
	private PersonBean creator = null;

	/**
	 * List of contributors to the artifact.
	 * 
	 */
	private Set<PersonBean> contributors = new HashSet<PersonBean>();

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getMimeType() {
		return mimeType;
	}

	public void setMimeType(String contentType) {
		this.mimeType = contentType;
	}

	public long getSize() {
		return size;
	}

	public void setSize(long size) {
		this.size = size;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Date getLastModifiedDate() {
		return lastModifiedDate;
	}

	public void setLastModifiedDate(Date date) {
		this.lastModifiedDate = date;
	}

	public PersonBean getCreator() {
		return creator;
	}

	public void setCreator(PersonBean creator) {
		this.creator = creator;
	}

	/**
	 * Return the set of PersonBeans that represents those that are contributors
	 * to the artifact.
	 * 
	 * @return set of PersonBeans that represents all the users that have
	 *         contributes to the artifact.
	 */
	public Set<PersonBean> getContributors() {
		return this.contributors;
	}

	/**
	 * Set the set of PersonBeans that represents those that contributed to the
	 * artifact.
	 * 
	 * @param contributors
	 *            the set of contributors to the artifact.
	 */
	public void setContributors(Collection<PersonBean> contributors) {
		this.contributors.clear();
		if (contributors != null) {
			this.contributors.addAll(contributors);
		}
	}

	/**
	 * Add the contributor to the set of contributors to the artifact.
	 * 
	 * @param contributor
	 *            the PersonBean of the contributor to be added.
	 */
	public void addContributor(PersonBean contributor) {
		if (contributor != null) {
			this.contributors.add(contributor);
		}
	}

	/**
	 * Remove the contributor from the set of contributors to the worflow tool.
	 * 
	 * @param contributor
	 *            the PersonBean of the contributor to be removed.
	 */
	public void removeContributor(PersonBean contributor) {
		this.contributors.remove(contributor);
	}

	public String getDcmiType() {
		return dcmiType;
	}

	public void setDcmiType(String dcmiType) {
		this.dcmiType = dcmiType;
	}
}
